@@ -410,7 +410,7 @@ static void fp_from_hash(element_ptr a, int len, void *data)
     mpz_t z;
 
     mpz_init(z);
-    mpz_import(z, len, 1, 1, 0, 0, data);
+    mpz_import(z, len, -1, 1, -1, 0, data);
     fp_set_mpz(a, z);
     mpz_clear(z);
 }
@@ -427,6 +427,27 @@ static int fp_cmp(element_ptr a, element_ptr b)
     }
 }
 
+static int fp_sgn_odd(element_ptr a)
+{
+    fp_field_data_ptr p = a->field->data;
+    dataptr ad = a->data;
+    if (!ad->flag) return 0;
+    return ad->d[p->limbs - 1] & 1 ? 1 : -1;
+}
+
+static int fp_sgn_even(element_ptr a)
+{
+    fp_field_data_ptr p = a->field->data;
+    dataptr ad = a->data;
+    if (!ad->flag) return 0;
+    mp_limb_t sum[p->limbs];
+
+    int carry = mpn_add_n(sum, ad->d, ad->d, p->limbs);
+    if (carry) return 1;
+    return mpn_cmp(sum, p->primelimbs, p->limbs);
+}
+
+
 static int fp_is_sqr(element_ptr a)
 {
     dataptr ad = a->data;
@@ -476,6 +497,7 @@ static int fp_from_bytes(element_t a, unsigned char *data)
     mpz_set_ui(z, 0);
 
     ptr = data;
+    //TODO: could be improved if needed
     n = a->field->fixed_length_in_bytes;
     for (i=0; i<n; i++) {
 	if (*ptr) ad->flag = 2;
@@ -484,7 +506,7 @@ static int fp_from_bytes(element_t a, unsigned char *data)
 	ptr++;
 	mpz_add(z, z, z1);
     }
-    if (ad->flag) fp_set_mpz(a, z);
+    if (ad->flag) from_mpz(a, z);
     mpz_clear(z);
     mpz_clear(z1);
     return n;
@@ -517,6 +539,7 @@ void field_init_faster_fp(field_ptr f, mpz_t prime)
     f->pow_mpz = fp_pow_mpz;
     f->neg = fp_neg;
     f->cmp = fp_cmp;
+    f->sign = mpz_odd_p(prime) ? fp_sgn_odd : fp_sgn_even;
     f->invert = fp_invert;
     f->random = fp_random;
     f->from_hash = fp_from_hash;
