@@ -1,8 +1,8 @@
+//requires
+// * gmp.h
 #ifndef RANDOM_H
 #define RANDOM_H
 
-#include <gmp.h>
-
 /*@manual pbcrandom
 Uses ''filename'' as a source of random bytes. For example,
 on Linux one might use <filename>/dev/random</filename>.
